//Packet Manager Module

module PacketManager(
										data_in,
										packet_out,
										timeout,
										dataAvailable_in,
										reset,
										packet_ready_out,
										idle,
										last_packet,
										fast_clock,
										timeout_in,
										timeout_count,
										data_index_out
										); 
							
input wire timeout_in;			
input wire [7:0] data_in;
output wire [79:0] packet_out;
output wire packet_ready_out;
output reg timeout;
input wire reset;
input wire dataAvailable_in;
input wire idle;
reg [3:0] data_index;
reg [79:0] packet;
reg packet_ready;
output reg [79:0] last_packet;
input wire fast_clock;
reg [79:0] sys_time;
assign packet_out = packet;
assign packet_ready_out = packet_ready;
output wire [24:0] timeout_count;
output wire [3:0] data_index_out;

assign data_index_out = data_index;
assign timeout_count = (sys_time - start_time) << 104;

reg [127:0] start_time, end_time;
reg inProgress;

always @(posedge fast_clock) begin
	if(reset) begin
		sys_time <= 0;
	end
	else begin
		sys_time <= sys_time + 1;
		if( ((sys_time - start_time) > 128'd5000000 ) && (inProgress == 1'b1)) begin
			timeout <= 1'b1;
			
		end
		else begin
			timeout <= 1'b0;
		end
	end
end


always @ (posedge dataAvailable_in, posedge reset) begin
	if(reset) begin
		data_index <= 4'd0;
		packet_ready <= 1'b0;
		packet <= 79'd0;
		last_packet <= 79'd0;
		start_time <= 0;
		end_time <= 0;
		inProgress <= 0;
	end
	else begin
		/*if(timeout_in) begin
			packet <= {72'd0,data_in};
			inProgress <= 1;
			start_time <= sys_time;
			packet_ready <= 1'b0;
			data_index <= 4'b0001;
		end
			else */begin
			if(data_index < 4'd9) begin
				data_index <= data_index + 4'b0001;
				packet_ready <= 1'b0;
				if(data_index == 4'd0) begin
					packet <= {72'd0,data_in};
					inProgress <= 1;
					start_time <= sys_time;
				end
				else begin
					packet <= (packet << 8) | {72'd0,data_in};
				end
			end
			else begin
				packet_ready <= 1'b1;
				packet <= (packet << 8) | {72'd0,data_in};
				last_packet <= (packet << 8) | {72'd0,data_in};
				data_index <= 4'd0;
				inProgress <= 0;
			end
		end
		
		
		
	end
end

										
										
endmodule
