module SerialReceiver_improved_fastClock( 
	input CLOCK_50,
	input  UART_RXD,
	output [7:0] lastReceived,
	input reset,
	output errorFlag_out,
	output dataReady_out,
	output [7:0] dataCounter,
	output [17:0] bigCounter,
	output [6:0] errs,
	output [3:0] state_out,
	output [8:0] error_check,
	input timeout,
	input baud_clock,
	input baud_8_clock
	);
	
	assign errs = errs_reg;
	assign bigCounter = bigCounter_reg;
	reg [6:0] errs_reg;
	reg [12:0] waiter = 13'b0000000000000; //counter to reduce clock rate
	
	reg clk_new = 0; //changes the state of an led every time a bit is sent
	
	reg [7:0] rxd;
		
	reg [3:0] currBit;
	reg [8:0] errorCheck;
	reg [3:0] errorBit;
	
	assign lastReceived = rxd;
	reg [17:0] bigCounter_reg;
	reg [3:0] receiveState;
	
	assign error_check = errorCheck;
	
	parameter [3:0] waitForStartBit = 4'd0,
					collectData = 4'd1,
					waitForStopBit = 4'd2,
					error_1 = 4'd3;
					
	assign state_out = receiveState[3:0];
					
	reg errorFlag;
	reg dataReady;
	
	assign dataReady_out = dataReady;
	assign errorFlag_out = errorFlag;
	
	reg [7:0] dataCounter_reg;
	assign dataCounter = dataCounter_reg;
	
	reg [4:0] bitClock;
	wire internalBaudClock;
	
	assign internalBaudClock = (bitClock==4'd10);

	always @(posedge CLOCK_50)
	if(receiveState==waitForStartBit)
		bitClock <= 4'b0000;
	else
	if(baud_8_clock)
		bitClock <= {bitClock[2:0] + 4'b0001} | {bitClock[3], 3'b000};
		
	reg [1:0] RxD_sync_inv;
	always @(posedge CLOCK_50) if(baud_8_clock) RxD_sync_inv <= {RxD_sync_inv[0], ~UART_RXD};
	// we invert RxD, so that the idle becomes "0", to prevent a phantom character to be received at startup

	reg [1:0] RxD_cnt_inv;
	reg RxD_bit_inv;

	always @(posedge CLOCK_50)
	if(baud_8_clock)
	begin
		if( RxD_sync_inv[1] && RxD_cnt_inv!=2'b11) RxD_cnt_inv <= RxD_cnt_inv + 2'h1;
		else 
		if(~RxD_sync_inv[1] && RxD_cnt_inv!=2'b00) RxD_cnt_inv <= RxD_cnt_inv - 2'h1;

		if(RxD_cnt_inv==2'b00) RxD_bit_inv <= 1'b1;//was 0
		else
		if(RxD_cnt_inv==2'b11) RxD_bit_inv <= 1'b0;//was 1
	end


	
	always @ (posedge CLOCK_50)
	begin
		if(reset) begin
			receiveState <= 4'd0;
			dataReady <= 0;
			errorFlag <= 0;
			currBit <= 0;
			rxd <= 0;
			dataCounter_reg <= 0;
			bigCounter_reg <= 0;
			errs_reg <= 0;
		end
		else begin
			if(receiveState == waitForStartBit && baud_8_clock) begin
				if(~RxD_bit_inv) begin
					currBit <= 4'd0;
					receiveState <= collectData;
					errorFlag <= 0;
					dataReady <= 0;
				end
			end
			if(receiveState != waitForStartBit && internalBaudClock && baud_8_clock) begin
				case(receiveState)
					collectData: begin
						rxd[currBit] <= RxD_bit_inv;
						currBit <= currBit + 4'd1;
						if(currBit == 4'd7) begin
							receiveState <= waitForStopBit;
						end
					end
					
					waitForStopBit: begin
						if(RxD_bit_inv) begin
							errorFlag <= 0;
							receiveState <= waitForStartBit;
							dataReady <= 1;
							dataCounter_reg <= dataCounter_reg + 8'd1;
							bigCounter_reg <= bigCounter_reg + 1;
							errorCheck <= 0;
							errorBit <= 0;
						end
						else begin
							errorFlag <= 1;
							errs_reg <= errs_reg + 1;
							receiveState <= error_1;
							errorCheck <= {rxd,RxD_bit_inv};
							errorBit <= 0;
							//dataReady <= 1;
						end
						
					end
					error_1: begin
						//it needs to resynch
						errorCheck <= {errorCheck[7:0], RxD_bit_inv};
						
						if(errorBit < 7) begin
							errorBit <= errorBit + 1;
							dataReady <= 0;
						end
						else begin
							if(errorCheck == 9'h1FF) //all idle bits
							begin
								receiveState <= waitForStartBit;
								errorFlag <= 0;
								dataReady <= 1;
							end
							if(errorCheck[7] == 1'b0 && RxD_bit_inv == 1'b1) begin
								receiveState <= waitForStartBit;
								errorFlag <= 0;
								dataReady <= 1;
							end
						end
					end
				endcase
			end
		end
	end
	
endmodule
