module SerialTransmitter_improved( 
	input CLOCK_50,
	output  UART_TXD,
	input [7:0] toTransmit,
	input reset,
	output dataSent_out,
	input sendNew,
	input [17:0] SW,
	output idle,
	input goBackToIdle,
	output [7:0] lastTransmit,
	input timeout_send,
	input baud_clock,
	output sendInProgress,
	output reg [4:0] transmitCount
	);
	
	reg [12:0] waiter = 13'b0000000000000; //counter to reduce clock rate
	
	reg clk_new = 0; //changes the state of an led every time a bit is sent
	
	reg [7:0] txd, last_txd;
		
	reg [3:0] currBit;
	
	reg [3:0] transmitState;
	
	parameter [3:0] idleWait = 4'd0,
					sendStartBit = 4'd1,
					sendData = 4'd2,
					sendStopBit = 4'd3;
					
	reg errorFlag;
	reg dataSent;
	reg txd_out;
	reg should_send;
	reg sendInProgressReg;
	
	assign idle = (transmitState == sendStopBit) ? 1'b1 : 1'b0;
	
	assign UART_TXD = txd_out;
	
	assign dataSent_out = dataSent;
	
	assign lastTransmit = last_txd;
	
	reg isTimeoutWaiting;
	
	//output reg [4:0] transmitCount;
	
		
	always @(posedge CLOCK_50)
	begin
		if(sendNew & ~reset & ((transmitState == idleWait) || (transmitState == sendStopBit))) begin
			dataSent <= 0;
			should_send <= 1'b1;
			isTimeoutWaiting <= 1'b0;
		end/*
		if(timeout_send & (transmitState == idleWait)) begin
			dataSent <= 0;
			//timeout_send <= 1'b1;
			isTimeoutWaiting <= 1'b1;
			//state <= idleWait;
			should_send <= 1'b1;
		end*/
		if(reset) begin
			transmitState <= 4'd0;
			dataSent <= 0;
			currBit <= 0;
			txd <= 0;
			txd_out <= 1;
			should_send <= 0;
			isTimeoutWaiting <= 0;
			sendInProgressReg <= 0;
			transmitCount <= 0;
		end
		else begin
			if (baud_clock) //9600 times a second (aprox)
			begin
				
				
				case(transmitState)
					idleWait: begin
						txd_out <= 1'b1;
						sendInProgressReg <= 1'b0;
						if(should_send) begin
						/*
							if(isTimeoutWaiting) begin
								txd <= 8'd128;
								last_txd <=  8'd128;
							end
							else*/ begin
								txd <= toTransmit;
								last_txd <= toTransmit;
							end
							transmitCount <= transmitCount + 1;
							transmitState <= sendStartBit;
							dataSent <= 0;
							should_send <= 0;
							
						end
					end
					
					sendStartBit: begin
						txd_out <= 1'b0; //the start bit
						transmitState <= sendData;
						currBit <= 4'd0;
						sendInProgressReg <= 1'b1;
					end
					
					sendData: begin
						txd_out <= txd[currBit];
						currBit <= currBit + 4'd1;
						if(currBit == 4'd7) begin
							transmitState <= sendStopBit;
						end
					end
					
					sendStopBit: begin
						txd_out <= 1'b1;
						dataSent <= 1;
						sendInProgressReg <= 1'b0;
						/*if(isTimeoutWaiting) begin
							if(~timeout_send) begin
								transmitState <= idleWait;
								isTimeoutWaiting <= 1'b0;
							end
						end
						else*/ begin
							if(/*goBackToIdle*/should_send) begin
								transmitState <= idleWait;
							end
						end
					end
					
										
				endcase
				
				
			end
			
		end
		
	end
endmodule
