//Peter Greczner, Instruction Decoder

module Decoder(instruction, ctrl_ALU, ctrl_BS, isBranch, isLoad, isStore, isJump, isDraw, isIMMOpp, SA, SB, SD, IMM, WE);

input wire [79:0] instruction;

output reg [3:0] ctrl_ALU;
output reg [3:0] ctrl_BS;
output reg isBranch;
output reg isLoad;
output reg isStore;
output reg isJump;
output reg isDraw;
output reg isIMMOpp;

output reg [4:0] SA, SB, SD;
output reg [31:0] IMM;
output reg WE;

parameter   ADD = 6'b100000,
            SUB = 6'b100001,
            SL  = 6'b100010,
            SR  = 6'b100011,
            OR  = 6'b100100,
            NOR = 6'b100101,
            AND = 6'b100110,
            NAND= 6'b100111,
            XOR = 6'b101000,
            ADDI= 6'b110101,
            SUBI= 6'b110110,
            SLI = 6'b110111,
            SRI = 6'b111000,
            LD  = 6'b110011,
            ST  = 6'b110100,
            NOT = 6'b111001,
            BGT = 6'b101001,
            BLT = 6'b101010,
            BGE = 6'b101011,
            BLE = 6'b101100,
            BEQ = 6'b101101,
            BNE = 6'b101110,
            BEZ = 6'b101111,
            BNEZ= 6'b110000,
            J   = 6'b110001,
            JR  = 6'b110010,
            JAL = 6'b111010,
            CIRCLE_FILL = 6'b000000,
            CIRCLE_DRAW = 6'b000001,
            SQUARE_FILL = 6'b000010,
            SQUARE_DRAW = 6'b000011,
            LINE_DRAW = 6'b000100,
            TEXT_WRITE = 6'b000101,
            FRAME_UPDATE = 6'b000110;
            
parameter   BS_bgt = 4'd0,
            BS_blt = 4'd1,
            BS_bge = 4'd2,
            BS_ble = 4'd3,
            //BS_ble = 4'd4,
            BS_beq = 4'd5,
            BS_bne = 4'd6,
            BS_bez = 4'd7,
            BS_bnez = 4'd8,
            BS_j = 4'd9,
            BS_jr = 4'd10,
            BS_jal = 4'd11,
            BS_nop = 4'd12;
            
parameter   ALU_add = 4'd0,
            ALU_sub = 4'd1,
            ALU_sl  = 4'd2,
            ALU_sr  = 4'd3,
            ALU_and = 4'd4,
            ALU_or  = 4'd5,
            ALU_xor = 4'd6,
            ALU_nand= 4'd7,
            ALU_not = 4'd8,
            ALU_nor = 4'd9,
            ALU_nop = 4'd10;

always @(instruction)
begin
    case(instruction[79:74])
        ADD: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_add;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        SUB: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_sub;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        SL: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_sl;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        SR: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_sr;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        OR: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_or;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        NOR: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_nor;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        AND: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_and;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        NAND: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_nand;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        XOR: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_xor;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        ADDI: begin
            SA = instruction[73:69];
            SD = instruction[68:64];
            SB = instruction[68:64];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            WE = 1;
            ctrl_ALU = ALU_add;
            IMM = instruction[63:32];
            ctrl_BS = BS_nop;
        end
        SUBI: begin
            SA = instruction[73:69];
            SD = instruction[68:64];
            SB = instruction[68:64];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            WE = 1;
            ctrl_ALU = ALU_sub;
            IMM = instruction[63:32];
            ctrl_BS = BS_nop;
        end
        SLI: begin
            SA = instruction[73:69];
            SD = instruction[68:64];
            SB = instruction[68:64];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            WE = 1;
            ctrl_ALU = ALU_sl;
            IMM = instruction[63:32];
            ctrl_BS = BS_nop;
        end
        SRI: begin
            SA = instruction[73:69];
            SD = instruction[68:64];
            SB = instruction[68:64];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            WE = 1;
            ctrl_ALU = ALU_sr;
            IMM = instruction[63:32];
            ctrl_BS = BS_nop;
        end
        ST: begin
            SA = instruction[73:69];
            SD = instruction[68:64];
            SB = instruction[68:64];
            isBranch = 0;
            isLoad = 0;
            isStore = 1;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            WE = 0;
            ctrl_ALU = ALU_add;
            IMM = instruction[63:32];
            ctrl_BS = BS_nop;
        end
        LD: begin
            SA = instruction[73:69];
            SD = instruction[68:64];
            SB = instruction[68:64];
            isBranch = 0;
            isLoad = 1;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            WE = 1;
            ctrl_ALU = ALU_add;
            IMM = instruction[63:32];
            ctrl_BS = BS_nop;
        end
        NOT: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 1;
            ctrl_ALU = ALU_not;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        BGT: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_bgt;
        end
        BLT: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_blt;
        end
        BGE: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_bge;
        end
        BLE: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_ble;
        end
        BEQ: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_beq;
        end
        BNE: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_bne;
        end
        BEZ: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_bez;
        end
        BNEZ: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 1;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_bnez;
        end
        J: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 1;
            isDraw = 0;
            isIMMOpp = 1;
            IMM = instruction[63:32];
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_j;
        end
        JR: begin
            SA = instruction[73:69];
            SB = 0;
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 1;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_jr;
            IMM = 0;
        end
        JAL: begin
            SA = instruction[73:69];
            SB = 0;
            SD = 5'd31;
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 1;
            isDraw = 0;
            isIMMOpp = 0;
            IMM = instruction[63:32];
            WE = 1;
            ctrl_ALU = ALU_add;
            ctrl_BS = BS_jr;
        end
        CIRCLE_FILL: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 1;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_nop;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        CIRCLE_DRAW: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 1;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_nop;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        SQUARE_FILL: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 1;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_nop;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        SQUARE_DRAW: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 1;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_nop;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        LINE_DRAW: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 1;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_nop;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        TEXT_WRITE: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 1;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_nop;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        FRAME_UPDATE: begin
            SA = instruction[73:69];
            SB = instruction[68:64];
            SD = instruction[63:59];
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 1;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_nop;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
        default: begin
            SA = 0;
            SB = 0;
            SD = 0;
            isBranch = 0;
            isLoad = 0;
            isStore = 0;
            isJump = 0;
            isDraw = 0;
            isIMMOpp = 0;
            WE = 0;
            ctrl_ALU = ALU_and;
            IMM = 0;
            ctrl_BS = BS_nop;
        end
    endcase
end

endmodule


/*

Instruction - 79:74

Drawing Functions: 000XXX

Other Functions:
*Add, subtract, shift, or, nor, and, nand, xor
*BGT, BLT, BGTE, BLTE, BEQ, BNE, J, JR, BEZ, BNEZ
*LD, ST
*(SLT, SGT)












*/

