//Request Manager

module requestManager(
	request,
	request_clk,
	control_clk,
	reset,
	isAvail,
	avail_select,
	request_avail,
	oinstr, ocolor, oxstart, oxend_width, oystart, oyend_height, oextra1, oextra2,
	cmd,
	send_ACK_NACK,
	transmit_idle,
	request_state,
	took_packet,
	VGA_SYNC,
	isItSendingACK
);

input wire [79:0] request;
input wire request_clk;
input wire control_clk;
input wire reset;
input wire isAvail;
input wire request_avail;
input wire transmit_idle;
input wire VGA_SYNC;
input wire isItSendingACK;

output wire took_packet;
reg took_packet_reg;
assign took_packet = took_packet_reg;

output wire send_ACK_NACK;
reg ack_nack;
assign send_ACK_NACK = ack_nack;

output reg cmd;

output reg [3:0] avail_select;

reg [5:0] instr;
reg [11:0] color;
reg [9:0] xstart, xend_width;
reg [8:0] ystart, yend_height;
reg [7:0] extra1, extra2;

output reg [5:0] oinstr;
output reg [11:0] ocolor;
output reg [9:0] oxstart, oxend_width;
output reg [8:0] oystart, oyend_height;
output reg [7:0] oextra1, oextra2;


//reg [2:0] request_state;
output reg [2:0] request_state;
parameter 	check_availability = 3'd0,
			issue = 3'd1,
			check_finished = 3'd2,
			send_ack = 3'd3,
			wait_ready = 3'd4,
			safety = 3'd5,
			safety2 = 3'd6,
			safety3 = 3'd7;

always @ (posedge request_clk)
begin
	instr <= request[79:74]; //Instruction
	color <= request[73:62]; //Color
	xstart <= request[61:52]; //x_start
	ystart <= request[51:43]; //y_start
	xend_width <= request[42:33]; //x_end/width
	yend_height <= request[32:24]; //y_end/height
	extra1 <= request[23:16]; //extra_1
	extra2 <= request[15:8]; //extra_2
	
	
end



always @ (posedge control_clk)
begin
	if(reset) begin
		request_state <= wait_ready;
		cmd <= 1'b0;
		ack_nack <= 1'b0;
		avail_select <= 4'd15;
		took_packet_reg <= 1'b0;
	end
	else begin
		if(VGA_SYNC) begin
			case(request_state)
			
				wait_ready: begin
					ack_nack <= 1'b0;
					if(request_avail) begin
						request_state <= safety3;
						avail_select <= request[77:74];
							oinstr <= instr;
						ocolor <= request[73:62];
						oxstart <= request[61:52]; //x_start
						oystart <= request[51:43]; //y_start
						oxend_width <= request[42:33]; //x_end/width
						oyend_height <= request[32:24]; //y_end/height
						oextra1 <= request[23:16]; //extra_1
						oextra2 <= request[15:8]; //extra_2
						
						took_packet_reg <= 1'b1;
					end
				end
				
				safety3: begin
					request_state <= check_availability;
				end
				
				check_availability: begin
					if(isAvail) begin
						request_state <= issue;
						cmd <= 1'b1;
					end
					
				end
				
				issue: begin
					//cmd <= 1'b1;
					request_state <= safety;
				end
				
				safety: begin
					request_state <= check_finished;
					cmd <= 1'b0;
				end
				
				check_finished: begin
					if(isAvail) begin
						request_state <= safety2;
						ack_nack <= 1'b1;
						//avail_select <= 4'd15; //idle state
					end
				end
				
				safety2: begin
					if(isItSendingACK == 1'b1) begin
						request_state <= send_ack;
						ack_nack <= 1'b0;
					end
					
				end
				
				send_ack: begin
					took_packet_reg <= 1'b0;
					ack_nack <= 1'b0;
					if(~request_avail) begin
						request_state <= wait_ready;
					end
				end
				
				default: begin
					request_state <= wait_ready;
				end
			endcase
		end
	end
	//see if available
	//issue command
	//see iff done
	//acknowledge done
end





endmodule

