module square_draw(x1,y1,width, height, VGA_SYNC, VGA_CLK, 
									data_reg, reset, color, command,
									available,
									my_id,
									id_req,
									xAddr, yAddr,
									write_clk,
									is_write_finished
									);
									
input wire signed [10:0] x1,y1;
input wire signed [10:0] width, height;
input wire VGA_SYNC;
input wire VGA_CLK;
input wire [3:0] my_id, id_req;
//input wire [9:0] Coord_X, Coord_Y;
//output reg we; // we for SRAM
//output reg [17:0] addr_reg; //memory address register for SRAM
output reg [15:0] data_reg; //memory data register  for SRAM
//input wire draw;
input wire reset;
input wire [15:0] color;
input wire [1:0] command;
output wire available;
reg available_reg;
assign available = available_reg;

output reg write_clk;
input wire is_write_finished;

output reg [9:0] xAddr;
output reg [8:0] yAddr;

reg signed  [10:0] x, x_end;
reg signed  [10:0] y, y_end;
//reg signed  [10:0] p;
//reg signed [10:0] draw_x, draw_y;

reg [4:0] draw_state, next_state;
parameter start = 5'd0,
			loop = 5'd1,
			finish = 5'd2,
			north = 5'd3,
			south = 5'd4,
			east = 5'd5,
			west = 5'd6,
			write_state = 5'd7;

always @ (posedge VGA_CLK) begin
	if(reset) begin
		draw_state <= finish;
		//clear the screen
		//addr_reg <= {Coord_X[9:1],Coord_Y[9:1]} ;	// [17:0]
		//we <= 1'b0;								//write some memory
		data_reg <= 16'b0;						//write all zeros (black)
		available_reg <= 1'b0;
		write_clk <= 1'b0;
	end
	else if (VGA_SYNC) begin
		case (draw_state)
		
			write_state: begin
				if(is_write_finished) begin
					draw_state <= next_state;
					write_clk <= 1'b0;
				end
			end
			
			north: begin
				//we <= 1'b0;
				if(x < x_end) begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= north;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					x <= x + 11'd1;
				end
				else begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= south;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					y <= y_end;
					x <= x1;
					//draw_state <= south;
				end
			end
			
			south: begin
				if(x < x_end) begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= south;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					x <= x + 11'd1;
				end
				else begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= east;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					//draw_state <= east;
					x <= x1;
					y <= y1;
				end
			end
			
			east: begin
				if(y < y_end) begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= east;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					y <= y + 11'd1;
				end
				else begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= west;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					//draw_state <= west;
					x <= x_end;
					y <= y1;
				end
			end
			
			west: begin
				if(y < y_end) begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= west;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					y <= y + 11'd1;
				end
				else begin
					//addr_reg <= {x[8:0],y[8:0]};
					xAddr <= x[9:0];
					yAddr <= y[8:0];
					next_state <= finish;
					draw_state <= write_state;
					write_clk <= 1'b1;
					data_reg <= color;
					//draw_state <= finish;
				end
			end
			
			finish: begin
				if(command[1] && (my_id == id_req)) begin
					draw_state <= north;
					x <= x1;
					y <= y1;
					x_end <= x1 + width;
					y_end <= y1 + height;
					available_reg <= 1'b0;
				end
				else begin
					available_reg <= 1'b1;
				end
				//we <= 1'b1;
			end
			
			default: begin
				//we <= 1'b1;
				draw_state <= finish;
			end
		endcase
	end
	else
	begin
		//addr_reg <= {Coord_X[9:1],Coord_Y[9:1]} ;
		//we <= 1'b1;
	end
end



endmodule
